module main;
parameter STEP=10;
parameter debug=0;
integer i,j;
reg p_reset, m_clock, extint;
reg [15:0] imem [0:1023];
reg [15:0] dmem [0:1023];
reg [15:0] inst,datai;
wire [15:0]	datao,iadrs,adrs;

wire	inst_read,inst_write,
	memory_read,memory_write,wb,hlt;

snxp cpu(p_reset,m_clock,inst,datai,
	datao,iadrs,adrs,
	extint, inst_read,
	memory_read,memory_write,hlt);


always #(STEP/2) m_clock=~m_clock;

always @(negedge m_clock)
begin
if(debug) begin
 $display("ntif:%b dtif:%b etif:%b mretry:%b",
   cpu.ntif, cpu.dtif, cpu.etif, cpu.mretry);
 $display("nretry:%b btbv:%b btb:%x fq0v:%b fq0:%x",
   cpu.nretry, cpu.btbv, cpu.btb, cpu.fq0v, cpu.fq0);
 $display("stall_req:%b dclasel1:%b dclasel2:%b dtR2:%x dtR3:%x",
   cpu.stall_req, cpu.clasel1, cpu.clasel2, cpu.dtR2, cpu.dtR3);
 $display("ru1:%b ru2:%b idarg:%x", cpu.ru1, cpu.ru2, cpu.idarg);
 $display("useR2:%b useR3:%b  writeR1:%b writeR2:%b",
   cpu.useR2, cpu.useR3, cpu.writeR1, cpu.writeR2);
 $display("epc:%x eop:%x er2f:%x eop1:%x eop2:%x alusel:%b aluo:%x eusemap:%b",
   cpu.epc, cpu.eop, cpu.er2f, cpu.eop1, cpu.eop2, cpu.alusel, cpu.aluo, cpu.eusemap);
 $display("mpc:%x mop:%x malu:%x mretry:%b brtaken:%b mforcenop:%b",
   cpu.mpc, cpu.mop, cpu.malu, cpu.mretry, cpu.br_taken, cpu.mforcenop);
 $display("mretryreq:%b musemap:%b mhit:%b mldbhit:%b mstbhit:%b ",
   cpu.mretryreq, cpu.musemap, cpu.mhit, cpu.mldbhit, cpu.mstbhit);
 $display("mnop0:%b mnop1:%b mnop2:%b musemap:%b ",
   cpu.mnop0, cpu.mnop1, cpu.mnop2, cpu.musemap);
 $display("mstb0v:%b mstb1v:%b mstb0d:%x mstb1d:%x ",
   cpu.mstb0v, cpu.mstb1v, cpu.mstb0d, cpu.mstb1d);
 $display("mstb0a:%x mstb1a:%x ",
   cpu.mstb0a, cpu.mstb1a);
 $display("mldb0v:%b mldb1v:%b mldb0d:%x mldb1d:%x ",
   cpu.mldb0v, cpu.mldb1v, cpu.mldb0d, cpu.mldb1d);
 $display("mldb0r:%b mldb0a:%x mldb1r:%b mldb1a:%x ",
   cpu.mldb0r, cpu.mldb0a, cpu.mldb1r, cpu.mldb1a);
 $display("wop:%b walu:%x grwrite:%b", cpu.wop, cpu.walu, cpu.gr.write);
 $display("gr.in:%x gr.inadr:%b gr.outadr:%b gr.outbadr:%b",
   cpu.gr.in, cpu.gr.inadr, cpu.gr.outadr, cpu.gr.outbadr);
 $display("====================================");
 end
if(memory_read)
  datai <= dmem[adrs];
if (memory_write)
   dmem[adrs] <= datao;

if(inst_read)
  begin
    inst <= imem[iadrs];
  end

if(hlt)
  begin
   $display("\nHALTED at %8d clock", $time/STEP);
   for(i=0; i<128; i=i+8)
      begin
       $write("%4x: ",i);
       for(j=0; j<8; j=j+1)
         $write("%x ",dmem[i+j]);
       $display;
      end
   $finish;
  end
end


initial begin
 $readmemh("snx.mem", imem);
 m_clock=0;
 extint=0;
 p_reset=1;
 $monitor("PC:%x OP:%x:%x:%x $0:%x $1:%x $2:%x $3:%x",
 cpu.pc, cpu.op, cpu.eop, cpu.mop, cpu.gr.r0,cpu.gr.r1,cpu.gr.r2,cpu.gr.r3);
 #(STEP) p_reset=0;
 #(60000*STEP+(STEP/2)) $finish;
end
endmodule

